@@ -26,6 +26,7 @@
 #' as (1-alpha)/2||.||_2^2+alpha||.||_1. 
 #' alpha=1 is the lasso penalty, and alpha=0 the ridge penalty. Default to 0.
 #' Same as in the glmnet package.
+#' @param print Default to TRUE.
 #' @return a list containing the fitted parameters.
 #' @references Rabiner, Lawrence R. "A tutorial on hidden Markov models and 
 #' selected applications in speech recognition." Proceedings of the 
@@ -41,9 +42,9 @@
 #' data(finance)
 #' x <- data.matrix(finance)
 #' #log return
-#' y <- x[-1,]
+#' y <- x[-1,-51]
 #' for(i in 2:nrow(x)){
-#'  y[i-1,] <- log(x[i,]) - log(x[i-1,])
+#'  y[i-1,] <- log(x[i,-51]) - log(x[i-1,-51])
 #' }
 #' #annualize the log return
 #' y <- y * 252 
@@ -84,21 +85,26 @@
 #' st3 <- viterbi.hmm(y=y,mod=fit3)
 #' sp3 <- smooth.hmm(y=y,mod=fit3)
 #' }
+#' @useDynLib rarhsmm, .registration = TRUE
+#' @importFrom Rcpp evalCpp
+#' @importFrom graphics points
+#' @importFrom stats rnorm
+#' @importFrom glmnet glmnet
 #' @export
 em.hmm <- function(y, mod, ntimes=NULL, 
                    tol=1e-4, maxit=100, arp=0, 
-                   cov.shrink=0, auto.lambda=0, auto.alpha=0){
+                   cov.shrink=0, auto.lambda=0, auto.alpha=0,print=TRUE){
   if(arp==0) 
-          result <- em.mvn(y, mod, ntimes, tol, maxit, cov.shrink)
+          result <- em.mvn(y, mod, ntimes, tol, maxit, cov.shrink,print)
   if(arp>0) 
     result <- em.mvnarp(y, mod, ntimes, tol, maxit, arp, cov.shrink,
-                        auto.lambda, auto.alpha)
+                        auto.lambda, auto.alpha,print)
   return(result)
 }
 
 ############################################
 #em algorithm for multivariate normal
-em.mvn <- function(y, mod, ntimes, tol, maxit,cov.shrink){
+em.mvn <- function(y, mod, ntimes, tol, maxit,cov.shrink,print){
   
   ns <- nrow(y)
   if(is.null(ntimes)) ntimes <- ns
@@ -111,87 +117,24 @@ em.mvn <- function(y, mod, ntimes, tol, maxit,cov.shrink){
   mu <- mod$mu
   sigma <- mod$sigma
   
-  #initialization
-  k1 <- (2*3.1415926)^(-p/2)
-  
   #recursion
   loglik <- 0
   for(iter in 1:maxit){
-    
     oldlik <- loglik
+    nodeprob <- getnodeprob_nocov_mvn(y, mu, sigma, K, p, 0, 0)
+    fb <- forwardbackward(Pi,P,nodeprob,ns,ntimes)
+
+    colsumXi <- fb$colsumxi
+    Gamma <- fb$Gamma
+    Gammasum <- fb$colsumgamma
+    loglik <- fb$loglik
+   
+    if(iter <= 1) likbase <- loglik
     
-    #get the cholesky factor: sigma = L%*%L^t
-    L <- lapply(1:K, function(k) t(chol(sigma[[k]])))
-    Lt <- lapply(L,t)
-    k2 <- sapply(1:K,function(k) k1/prod(diag(L[[k]]))) #sqrt(det(sigma))
-    count <- 0
-    
-    Gamma <- NULL
-    Gammasum <- matrix(0,1,K)
-    oldlik <- loglik
-    colsumXi <- rep(0, K*K)
-    
-    #for each subject
-    loglik <- 0 
-    
-    for(n in 1:N){
-      
-      alpha <- matrix(0,ntimes[n],K)
-      beta <- matrix(0,ntimes[n],K)
-      gamma <- matrix(0,ntimes[n],K)
-      B <- matrix(0,ntimes[n],K)
-      
-      Scale <- rep(0, ntimes[n])
-      Xi <- matrix(0, ntimes[n]-1, K*K)
-    
-    #state-dependent probs
-    for(i in 1:ntimes[n]){
-      for(j in 1:K){
-        diff <- mu[[j]] - y[count+i,]
-        B[i,j] <- k2[[j]]*exp(-0.5*t(diff)%*%
-                           backsolve(Lt[[j]],forwardsolve(L[[j]],diff)))
-      }
-    }
-    
-    ####E-step
-    #forward-backward
-    
-    scale <- rep(0, ntimes[n])
-    alpha[1,] <- Pi*B[1,]
-    scale[1] <- sum(alpha[1,])
-    alpha[1,] <- alpha[1,]/scale[1]
-    
-    for(i in 2:ntimes[n]){
-      alpha[i,] <- (alpha[i-1,]%*%P)*B[i,]
-      scale[i] <- sum(alpha[i,])
-      alpha[i,] <- alpha[i,] / scale[i]
-    }
-    
-    beta[ntimes[n],] <- rep(1/K,K)/scale[ntimes[n]]
-    for(i in (ntimes[n]-1):1)
-      beta[i,] <- (beta[i+1,] * B[i+1,]) %*% t(P) / scale[i]
-    
-    gamma <- alpha*beta
-    gamma <- gamma / rowSums(gamma) 
-    gammasum <- colSums(gamma) #updated
-    
-      xi <- matrix(0, ntimes[n]-1, K*K)
-      for(i in 1:(ntimes[n]-1)){
-        t <- P * ( alpha[i,] %*% t(beta[i+1,] * B[i+1,])  )
-        xi[i,] <- as.vector(t) / sum(t)
-      }
-      
-      Scale <- Scale + log(scale)
-      Gamma <- rbind(Gamma, gamma)
-      Gammasum <- Gammasum + gammasum
-      Xi <- Xi + xi
-      
-      colsumXi <- colsumXi + colSums(Xi)
-      loglik <- loglik + sum(Scale)
-      count <- count + ntimes[n]
-    }
-    
-    ##############
+    if(iter > maxit | 
+       (iter>1 & (loglik - likbase)/(oldlik - likbase) < 1 + tol) ) {
+      loglik <- oldlik; break;}
+    #############
     #M-step
     Mu <- matrix(0,K,p)
     Mu <- t(Gamma)%*%y
@@ -201,13 +144,7 @@ em.mvn <- function(y, mod, ntimes, tol, maxit,cov.shrink){
       
     sxi <- matrix(colsumXi, K,K)
     P <- sxi / rowSums(sxi)
-    Pi <- rep(0, K)
-    picount <- 0
-    for(n in 1:N){
-      Pi <- Pi + Gamma[picount+1,]
-      picount <- picount + ntimes[n]
-    }
-    Pi <- Pi / N
+    Pi <- Gamma[1,]
     
     sigma <- vector(mode="list", length=K)
     
@@ -227,14 +164,7 @@ em.mvn <- function(y, mod, ntimes, tol, maxit,cov.shrink){
       sigma[[l]] <- sigma[[l]]*(1-W) + diag(alpha,p)*W
     }
     
-    
-    
-    if(iter <= 1) likbase <- loglik
-    
-    cat("iteration ",iter,"; loglik = ", loglik, "\n")
-    
-    if(iter > maxit | 
-       (iter>1 & (loglik - likbase)/(oldlik - likbase) < 1 + tol) ) break;
+    if(print==TRUE) cat("iteration ",iter,"; loglik = ", loglik, "\n")
   }
   
   return(list(m=K, mu=mu, sigma=sigma, delta=Pi, gamma=P,
@@ -245,7 +175,7 @@ em.mvn <- function(y, mod, ntimes, tol, maxit,cov.shrink){
 #####################################################
 #####################################################
 em.mvnarp <- function(y, mod, ntimes, tol, maxit, arp,
-                      cov.shrink, auto.lambda, auto.alpha){
+                      cov.shrink, auto.lambda, auto.alpha,print){
   
   ns <- nrow(y)
   if(is.null(ntimes)) ntimes <- ns
@@ -259,109 +189,33 @@ em.mvnarp <- function(y, mod, ntimes, tol, maxit, arp,
   sigma <- mod$sigma
   auto <- mod$auto
   
-  #initialization
-  k1 <- (2*3.1415926)^(-p/2)
-  
   #recursion
   loglik <- 0
   for(iter in 1:maxit){
-    
     oldlik <- loglik
+    ################################
+    #nodeprob
+    ycov <- rbind(rep(0,p),y[-ntimes,])
+    autoarray <- array(as.numeric(unlist(auto)), dim=c(p,p,K))
+    muarray <- array(as.numeric(unlist(mu)), dim=c(1,p,K))
+    nodeprob <- getnodeprob_part2(y, ycov,autoarray,muarray,sigma,K,p)
     
-    #get the cholesky factor: sigma = L%*%L^t
-    L <- lapply(1:K, function(k) t(chol(sigma[[k]])))
-    Lt <- lapply(L,t)
-    k2 <- sapply(1:K,function(k) k1/prod(diag(L[[k]]))) #sqrt(det(sigma))
-    count <- 0
-    
-    Gamma <- NULL
-    Gammasum <- matrix(0,1,K)
-    oldlik <- loglik
-    colsumXi <- rep(0, K*K)
-    
-    #for each subject
-    loglik <- 0 
-    
-    for(n in 1:N){
-      
-      alpha <- matrix(0,ntimes[n],K)
-      beta <- matrix(0,ntimes[n],K)
-      gamma <- matrix(0,ntimes[n],K)
-      B <- matrix(0,ntimes[n],K)
-      
-      Scale <- rep(0, ntimes[n])
-      Xi <- matrix(0, ntimes[n]-1, K*K)
-      
-      #state-dependent probs
-      for(i in 1:ntimes[n]){
-        for(j in 1:K){
-          if(i==1)  diff <- mu[[j]] - y[count+i,]
-          else if(i<=arp)diff <- mu[[j]] + auto[[j]][,(p*arp-(i-1)*p+1):(p*arp),drop=FALSE]%*%
-              as.vector(t(y[(count+1):(count+i-1),])) - 
-               y[count+i,]
-          else diff <- mu[[j]] + auto[[j]]%*%
-              as.vector(t(y[(count+i-arp):(count+i-1),])) - 
-              y[count+i,]
-          B[i,j] <- k2[[j]]*exp(-0.5*t(diff)%*%
-                                  backsolve(Lt[[j]],forwardsolve(L[[j]],diff)))
-        }
-      }
-      
-      ####E-step
-      #forward-backward
-      alpha[1:arp,] <- rep(0,K) ########
-      beta[1:arp,] <- rep(0,K)
-      
-      scale <- rep(0, ntimes[n])
-      scale[1:arp] <- 0 ########
-      
-      alpha[arp+1,] <- Pi*B[arp+1,]
-      scale[arp+1] <- sum(alpha[arp+1,])
-      alpha[arp+1,] <- alpha[arp+1,]/scale[arp+1]
-      
-      for(i in (arp+2):ntimes[n]){
-        alpha[i,] <- (alpha[i-1,]%*%P)*B[i,]
-        scale[i] <- sum(alpha[i,])
-        alpha[i,] <- alpha[i,] / scale[i]
-      }
-      
-      beta[ntimes[n],] <- rep(1/K,K)/scale[ntimes[n]]
-      for(i in (ntimes[n]-1):(arp+1)) ##
-        beta[i,] <- (beta[i+1,] * B[i+1,]) %*% t(P) / scale[i]
-      
-      gamma <- alpha*beta
-      gamma <- gamma / pmax(rowSums(gamma),0.01) 
-      gammasum <- colSums(gamma) #updated
-      
-      xi <- matrix(0, ntimes[n]-1, K*K)
-      for(i in (1+arp):(ntimes[n]-1)){ ###############################
-        t <- P * ( alpha[i,] %*% t(beta[i+1,] * B[i+1,])  )
-        xi[i,] <- as.vector(t) / sum(t)
-      }
-      
-      Scale <- Scale + log(scale)
-      Gamma <- rbind(Gamma, gamma)
-      Gammasum <- Gammasum + gammasum
-      Xi <- Xi + xi
-      
-      colsumXi <- colsumXi + colSums(Xi)
-      loglik <- loglik + sum(Scale[-(1:arp)])#######
-      count <- count + ntimes[n]
-      
-    }
+    fb <- forwardbackward(Pi,P,nodeprob,ns,ntimes)
+    colsumXi <- fb$colsumxi
+    Gamma <- fb$Gamma
+    Gammasum <- fb$colsumgamma
+    loglik <- fb$loglik
+    if(iter <= 1) likbase <- loglik
     
+    if(iter > maxit | 
+       (iter>1 & (loglik - likbase)/(oldlik - likbase) < 1 + tol) ) {
+      loglik <- oldlik; break;}
     ##############
     #M-step
     
     sxi <- matrix(colsumXi, K,K)
     P <- sxi / rowSums(sxi)
-    Pi <- rep(0, K)
-    picount <- 0
-    for(n in 1:N){
-      Pi <- Pi + Gamma[picount+arp+1,] ######
-      picount <- picount + ntimes[n]
-    }
-    Pi <- Pi / N
+    Pi <- Gamma[1,]
     
     sigma <- vector(mode="list", length=K)
     auto <- vector(mode="list", length=K)
@@ -402,12 +256,7 @@ em.mvnarp <- function(y, mod, ntimes, tol, maxit, arp,
       sigma[[l]] <- sigma[[l]]*(1-W) + diag(alpha,p)*W
     }
     
-    if(iter <= 1) likbase <- loglik
-    
-    cat("iteration ",iter,"; loglik = ", loglik, "\n")
-    
-    if(iter > maxit | 
-       (iter>1 & (loglik - likbase)/(oldlik - likbase) < 1 + tol) ) break;
+    if(print==TRUE) cat("iteration ",iter,"; loglik = ", loglik, "\n")
   }
   
   return(list(m=K, mu=mu, sigma=sigma, delta=Pi, gamma=P, auto=auto, arp=arp,
